#region Using directives
using System;
using System.Text;
using System.Runtime.InteropServices;
using System.IO;
using GTA;
#endregion

namespace ScriptMods
{
    class Dragonsbreath : Script
    {
		//	member
        #region script parameters
        string configFilePath = String.Format("{0}\\scripts\\dragonsbreath.ini", Game.InstallFolder.ToString());
        float fRadius;
        bool bRenderExplosions;
        float fPower;
        bool bExtraDamage;
        float fCameraShake;
        Ped[] peds;
        string tmp;
        bool bRunScript = true;	//	on / off
        bool bUpdated = false;	//	config status
        bool bFireproofed = true;
		#if DEBUG
		int db_victims = 0;
		int db_items = 0;
		int db_cyclesPM = 0;
		#endif
        #endregion

        //  ctor
        public Dragonsbreath()
        {
            Interval = 100;
            BindConsoleCommand("dragonsbreath", ConsoleCommandHandling);	//	for reading config parameters
            this.Tick += new EventHandler(Dragonsbreath_Tick);	//	core script
        }

        void Dragonsbreath_Tick(object sender, EventArgs e)
        {
            if (bRunScript)
            {
                if (bUpdated == false)
                {
                    ReadConfig(configFilePath);
                    bUpdated = true;
                }
                if (bFireproofed)
                {
                    Player.Character.FireDamageMultiplier = 0.0f;
                    Player.Character.MakeProofTo(false, true, false, false, false);
                }
                if (Player.CanControlCharacter && Player.Character.isAliveAndWell && !Player.Character.isInVehicle() && !Game.isMultiplayer)   //  controllable, alive, on foot)
                {
                    if (Player.Character.Weapons.Current.Slot == WeaponSlot.Shotgun)
                    {
                        peds = World.GetPeds(Player.Character.Position, fRadius);
                        foreach (Ped tmpPed in peds)
                        {
                            if (tmpPed != null)
                            {
                                if (!tmpPed.isOnFire && tmpPed.HasBeenDamagedBy(Player.Character) && tmpPed.HasBeenDamagedBy(Player.Character.Weapons.AnyShotgun) && !tmpPed.isInWater)
                                {
                                    if (tmpPed.isInVehicle())
                                    {
                                        tmpPed.LeaveVehicle();
                                    }
                                    if (bExtraDamage)
                                    {
                                        tmpPed.Health -= (tmpPed.Health / 2);
                                    }
                                    if (bRenderExplosions)
                                    {

                                        World.AddExplosion(tmpPed.Position, ExplosionType.Molotov, fPower, false, false, fCameraShake);
                                    }
                                    GTA.Native.Function.Call("START_CHAR_FIRE", tmpPed);
                                    tmpPed.DropCurrentWeapon();
                                    tmpPed.NoLongerNeeded();
                                }
                            }
                        }
                    }
                }
            }
        }

        #region methods
        private void ConsoleCommandHandling(ParameterCollection parameters)
        {
            switch (parameters.ToString(0))
            {
                case "-r":
                case "reload":
                    ReadConfig(configFilePath);
                    break;
                case "-n":
                case "newfile":
                    CreateCustomConfigFile(configFilePath);
                    break;
                case "off":
                    bRunScript = false;
                    Game.DisplayText("Dragonsbreath turned OFF!");
                    break;
                case "on":
                    bRunScript = true;
                    Game.DisplayText("Dragonsbreath turned ON!");
                    break;
                case "shotgun":
                    Game.DisableMaxAmmoLimit = true;
                    Player.Character.Weapons.BarettaShotgun.Ammo = 5000;
                    break;
                case "showconfig":
                case "-s":
                    if (Game.Console.isActive)
                    {
                        tmp = String.Format("Radius: {0}  Render explosions: {1}  Explosion power: {2}  Extra damage: {3}  Camera shake: {4}  Fireproofed: {5}", fRadius.ToString(), bRenderExplosions.ToString(), fPower.ToString(), bExtraDamage.ToString(), fCameraShake.ToString(), bFireproofed.ToString());
                        Game.Console.Print(tmp);
                    }
                    break;
                default:
                    tmp = string.Format("Error: Invalid argument:\t{0}", parameters.ToString(0));
                    Game.Console.Print(tmp);
                    break;
            }
        }

        private void ReadConfig(string path)
        {
            if (!File.Exists(path))
            {
                CreateCustomConfigFile(path);
            }
            IniReader ini = new IniReader(path);
            string tmp = ini.ReadIni("CONFIG", "Radius");
            if (!float.TryParse(tmp, out fRadius))
            {
                fRadius = 35.0f;
            }

            tmp = ini.ReadIni("CONFIG", "RenderExplosions");
            if (tmp == "1" || tmp == "true")
            {
                bRenderExplosions = true;
            }
            else
            {
                bRenderExplosions = false;
            }

            tmp = ini.ReadIni("CONFIG", "ExplosionPower");
            if (!float.TryParse(tmp, out fPower))
            {
                fPower = 1.0f;
            }

            tmp = ini.ReadIni("CONFIG", "ExtraDamage");
            if (tmp == "1" || tmp == "true")
            {
                bExtraDamage = true;
            }
            else
            {
                bExtraDamage = false;
            }

            tmp = ini.ReadIni("CONFIG", "CameraShake");
            if (!float.TryParse(tmp, out fCameraShake))
            {
                fCameraShake = 0.0f;
            }
            bUpdated = false;
            tmp = ini.ReadIni("CONFIG", "FireproofedPlayer");
            if (tmp == "1" || tmp == "true")
            {
                bFireproofed = true;
            }
            else
            {
                bFireproofed = false;
            }
        }

        private void CreateCustomConfigFile(string path)
        {
            if (File.Exists(path))
            {
                File.Delete(path);
            }
            using (StreamWriter sWriter = new StreamWriter(path))
            {
                sWriter.WriteLine("[CONFIG]");
                sWriter.WriteLine("Radius=35");
                sWriter.WriteLine("RenderExplosions=1");
                sWriter.WriteLine("ExplosionPower=1.0");
                sWriter.WriteLine("ExtraDamage=1");
                sWriter.WriteLine("CameraShake=0");
                sWriter.WriteLine("FireproofedPlayer=1");
            }
        }

        #endregion
    }

    public class IniReader
    {	
		//	member
        public string filePath;

        #region Import API
        [DllImport("kernel32")]
        private static extern int GetPrivateProfileString(string section, string keyEntry, string def, StringBuilder returnValue, int iSize, string filePath);
        #endregion

		//	ctor
        public IniReader(string path)
        {
            filePath = path;
        }

		//	wrapper method for reading config
        public string ReadIni(string section, string key)
        {
            StringBuilder tmpSB = new StringBuilder(255);
            int i = GetPrivateProfileString(section, key, "", tmpSB, 255, this.filePath);
            return tmpSB.ToString();
        }
    }
}